clc; close all; clear;

beta = 1;
alpha = -0.5;
N = 512;
L = 40;
n = 4;
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*L/N);
[phisrc,rhosrc] = cart2pol(xsrc,ysrc);

if n == 0
    Sthy = 1 + alpha*besselj(0,2*beta*rhosrc);
else
    Sthy = 1 + alpha*besselj(0,2*beta*rhosrc) ...
        + alpha*cos(n*pi/2)*cos(n*phisrc).*besselj(n,2*beta*rhosrc);
end
[x1,x2] = ndgrid((-N/2:N/2-1)*L/N); 
rho_d = sqrt((x1-x2).^2);
phi_d = atan2(zeros(size(x1)),(x1-x2));
rho_a = sqrt((x1+x2).^2);
phi_a = atan2(zeros(size(x1)),(x1+x2));
if n == 0
    Wthy = 1/2*(besselj(0,beta*rho_d) + 1j^n*cos(n*phi_d).*besselj(n,beta*rho_d) ...
        - 1j^n*(1-(-1)^n)*alpha^2/(2*(1+sqrt(1-alpha^2)))*cos(n*phi_d).*besselj(n,beta*rho_d)) ...
        + alpha/2*(besselj(0,beta*rho_a) + cos(n*pi/2)*cos(n*phi_a).*besselj(n,beta*rho_a));
else
    Wthy = 1/1*(besselj(0,beta*rho_d) + 1j^n*cos(n*phi_d).*besselj(n,beta*rho_d) ...
        - 1j^n*(1-(-1)^n)*alpha^2/(2*(1+sqrt(1-alpha^2)))*cos(n*phi_d).*besselj(n,beta*rho_d)) ...
        + alpha/1*(besselj(0,beta*rho_a) + cos(n*pi/2)*cos(n*phi_a).*besselj(n,beta*rho_a));
end    

numtrials = 100e3;
if n == 0
    v_theta = 2*pi*rand(1,numtrials);
else
    v = linspace(0,2*pi,360*100);
    CDF_v_theta = v/(2*pi).*(1 + sinc(1/pi*n*v));
    v_theta = interp1(CDF_v_theta,v,rand(1,numtrials),'pchip');
end
v_mag = ones(1,numtrials);
[vx,vy] = pol2cart(v_theta,v_mag);
SsimG = zeros(N);
WsimG = zeros(N);
for ii = 1:numtrials
    
    disp(ii)
    
    U = 1/sqrt(2)*( sqrt(1+sqrt(1-alpha^2))*exp(1j*beta*(xsrc*vx(ii) + ysrc*vy(ii))) ...
        + alpha/sqrt(1+sqrt(1-alpha^2))*exp(-1j*beta*(xsrc*vx(ii) + ysrc*vy(ii))) );
    
    SsimG = SsimG + abs(U).^2;
    SrmseG(ii) = sqrt(1/N^2*sum(sum((SsimG/ii - Sthy).^2)));
    ScorrG(ii) = corr2(SsimG/ii,Sthy);
    %%%%% W(x1,0;x2,0) %%%%%
    WsimG = WsimG + (U(N/2+1,:).')*conj(U(N/2+1,:));
    %%%%%%%%%%%%%%%%%%%%%%%%
    
end
SsimG = SsimG/numtrials;
WsimG = WsimG/numtrials;

save('HO_Dark_Antidark_Sim_Data.mat');